/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sidescroller;

import org.newdawn.slick.Image;
import org.newdawn.slick.geom.Circle;
import org.newdawn.slick.geom.Shape;
import org.newdawn.slick.geom.Vector2f;

/**
 *
 * @author sam
 */
public class Canon extends CollidableImageObject {

    /** the rate this canon fires at.*/
    private int fireRate;
    /** direction this canon fires in. */
    private int fireDirection;

    /**
     * each time fire() method is called, this field is raised by one.
     * when it is bigger than fireRate, the canon will fire and this field will
     * be reset.
     */
    private int lastShotFired;

    /** image of a canon ball. */
    private Image canonBallImage;

    /** speed of canon ball. */
    private int canonBallSpeed;

    /**
     * constructs new canon.
     * <p>
     * The fireDirection must be one of the following:
     * 1 for east,
     * 2 for south,
     * 3 for west,
     * 4 for north.
     *
     * A negative fireRate means, this canon will not fire.
     *
     * @param name name
     * @param canonImage image of the canon
     * @param canonBallImage image of a canon ball
     * @param position position
     * @param collisionShape collisionShape
     * @param fireRate the rate this canon fires at.
     * @param fireDirection direction this canon fires in
     * @param canonBallSpeed speed of canon ball
     */
    public Canon(String name, Image canonImage, Image canonBallImage,
            Vector2f position,
            Shape collisionShape, int fireRate, int fireDirection,
            int canonBallSpeed) {
        super(name, canonImage, position, collisionShape);
        this.fireDirection = fireDirection;
        this.fireRate = fireRate;
        this.canonBallImage = canonBallImage;
        this.canonBallSpeed = canonBallSpeed;
    }

    /**
     * each time this method is called, a counter will be raised.
     * When it is bigger than fireRate, the canon will fire and the counter will
     * be reset.
     * If the canon fires, the CanonBall will be returned and must be handled
     * from the outside from that point on.
     * If it did not fire, null will be returned.
     *
     * @return canon ball or null
     */
    public CanonBall fire() {
        lastShotFired++;

        CanonBall canonBall = null;
        if (fireRate <= lastShotFired) {
        Circle ballCollisionShape = new Circle(
                canonBallImage.getWidth() / 2f,
                canonBallImage.getHeight() / 2f,
                Math.max(canonBallImage.getWidth() / 2f,
                canonBallImage.getHeight() / 2f));
            canonBall = new CanonBall("canonBall", canonBallImage, position.copy(),
                    ballCollisionShape, fireDirection, canonBallSpeed);
            lastShotFired = 0;
        }
        return canonBall;
    }
}
