/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sidescroller;

import org.newdawn.slick.Image;
import org.newdawn.slick.geom.Shape;
import org.newdawn.slick.geom.Vector2f;

/**
 * This class represents a canon ball.
 * Canon balls are not subjected to gravity.
 *
 * @see Canon
 *
 * @author sam
 */
public class CanonBall extends CollidableImageObject {

    private int direction;
    private int speed;

    /**
     * constructs a new canon ball.
     * <p>
     * The direction must be one of the following:
     * 1 for east,
     * 2 for south,
     * 3 for west,
     * 4 for north.
     *
     * @param name name
     * @param image image
     * @param position position
     * @param collisionShape collisionShape
     * @param direction direction this canon fires in
     * @param speed distance the ball travels per move
     */
    public CanonBall(String name, Image image, Vector2f position,
            Shape collisionShape, int direction, int speed) {
        super(name, image, position, collisionShape);
        this.direction = direction;
        this.speed = speed;
    }

    /**
     * moves this canon ball in the direction it flies.
     * If it hits a wall it will return 1.
     * If it hits the player, it will return 2.
     * If it does not hit anything, it will return 0.
     *
     * @param map map
     * @param player player
     * @return 1 if wall hit, 2 if player hit, 0 otherwise
     */
    public int move(BlockMap map, Player player) {
        switch(direction) {
            case 1:
                 // move left
                return move(speed, 0, map, player);
            case 2:
                // move down
                return move(0, speed, map, player);
            case 3:
                // move right
                return move(-speed, 0, map, player);
            case 4:
                // move up
                return move(0, -speed, map, player);
            default:
                return 0;
        }
    }

    /**
     * moves this canon ball in the direction it flies.
     *
     * @param xDistance xDistance
     * @param yDistance yDistance
     * @param map map
     * @param player player
     * @return 1 if wall hit, 2 if player hit, 0 otherwise
     */
    private int move(int xDistance, int yDistance, BlockMap map,
            Player player) {
        position.set(position.x + xDistance, position.y + yDistance);
        if (map.isColliding(this)) {
            return 1;
        } else if (player.isCollidingWith(this)) {
            return 2;
        } else {
            return 0;
        }
    }
}
