/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sidescroller;

import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.geom.Shape;
import org.newdawn.slick.geom.Transform;
import org.newdawn.slick.geom.Vector2f;

/**
 * An <code>ImageObject</code> extended by the methods specified in
 * <code>ICollidableObject</code>.
 *
 * @author sam
 */
public class CollidableImageObject extends ImageObject
        implements ICollidableObject {

    private Shape collisionShape;

    /**
     * creates a new collidable image object.
     *
     * @param name name of the object
     * @param image image to be displayed
     * @param position position of the object
     * @param collisionShape shape used to calculate collision.
     * The position must not be set manuelly but will be adjusted when method
     * getCollisionShape is called
     */
    public CollidableImageObject(
            String name, Image image, Vector2f position, Shape collisionShape) {
        super(name, image, position);
        this.collisionShape = collisionShape;
    }

    /**
     * returns the general collision shape of this object.
     *
     * @return general collision shape
     */
    @Override
    public Shape getNormalCollisionShape() {
        return collisionShape;
    }

    /**
     * returns the collision shape of this object moved to its actual position.
     *
     * @return actual collision shape
     */
    @Override
    public Shape getCollisionShape() {
        return collisionShape.transform(
                Transform.createTranslateTransform(position.x, position.y));
    }

    /**
     * draws the <code>image</code> on the screen with its top left corner
     * at the given x/y coordinates.
     * If debug is enabled it will also draw its collision shape.
     *
     * @param graphics graphics context
     */
    @Override
    public void render(Graphics graphics) {
        image.draw(position.x, position.y);

        if (Settings.debug) {
            graphics.draw(getCollisionShape());
        }
    }

    /**
     * returns true if the collision shape of this object intersects with the
     * collision shape of the given object.
     * It uses the shapes provided by the
     * {@link #getCollisionShape() getCollisionShape} method.
     *
     * @param collidable collidable
     * @return is colliding?
     */
    @Override
    public boolean isCollidingWith(ICollidableObject collidable) {
        return collidable.getCollisionShape().intersects(
                this.getCollisionShape());
    }
}