/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sidescroller;

import java.awt.Point;
import java.util.ArrayList;
import java.util.List;

/**
 * This class represents a CollisionBlock that moves.
 * Everytime, its move method is called, its position will be set to the next
 * Point given in the points-List. If the end of the list is reached,
 * depending on the setting, it will either jump to the beginning of the list
 * or move backwards in the list.
 *
 * // TODO use this class. also, find a good way to represent
 *    moving things in tiledMap...
 *
 * @author sam
 */
public class CollisionBlockMoving extends CollisionBlock {

    private List<Point> points;
    private int currentPosition = 0;
    private boolean movingForwards = true;
    // private boolean jumptoStart = false;

    /**
     * constructs a new moving block.
     * The given list must contain at least one point.
     *
     * @param width width
     * @param higth higth
     * @param points points of path
     */
    public CollisionBlockMoving(int width, int higth, List<Point> points) {
        super(points.get(0).x, points.get(0).y, width, higth);
        this.points = points;
    }

    /**
     * constructs a new moving block.
     * The block will move smoothly at the given speed between the two given
     * points.
     * <P>
     * IMPORTANT:
     * startX must be smaller than endX and
     * startY must be smaller than endY
     *
     * @param width width
     * @param higth higth
     * @param startX startX
     * @param startY startY
     * @param endX endX
     * @param endY endY
     * @param speed speed
     */
    public CollisionBlockMoving(int width, int higth, int startX, int startY,
            int endX, int endY, int speed) {
        super(startX, startY, width, higth);

        // calculate line
        float m = (endY - startY) / ((float) (endX - startX)); // slope of line
        float c = endY - (m * endX); // y-intercept of line

        points = new ArrayList<Point>();
        points.add(new Point(startX, startY));
        int xOnLine = startX;
        int yOnLine = startY;
        while (xOnLine < endX && yOnLine < endY) {
            points.add(new Point(xOnLine, yOnLine));
            xOnLine += speed;
            yOnLine = (int) ((m * xOnLine) + c);
        }
    }

    /**
     * moves the block one step further on its path.
     * If the beginning or end of the path is reached, it will move backwards.
     */
    public void move() {
        if (movingForwards) {
            if (currentPosition + 1 < points.size()) {
                currentPosition++;
            } else {
                movingForwards = false;
            }
        } else {
            if (currentPosition - 1 > 0) {
                currentPosition--;
            } else {
                movingForwards = true;
            }
        }
        setPosition(points.get(currentPosition).x,
                points.get(currentPosition).y);
    }

    /**
     * sets position of collision shape.
     *
     * @param x new x
     * @param y new y
     */
    private void setPosition(int x, int y) {
        super.getCollisionShape().setLocation(x, y);
    }
}
