/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sidescroller;

import org.lwjgl.opengl.Display;
import org.lwjgl.opengl.DisplayMode;
import org.newdawn.slick.AppGameContainer;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.state.StateBasedGame;
import org.newdawn.slick.ScalableGame;

/**
 * // TODO general todos
 *
 * * functionality
 *   - pause in game
 *   - delete canon balls when they leave the map!
 *   - add slope collision "block".
 *   - fix error handling
 *   - options
 *   - more collectables
 *   - enemies that can walk
 *   - tests
 *   - sounds
 *   - more lvl
 *   - redo water movement (it is quite hard)
 *      (maybe just stop adding gravity after certain time)
 *   - implement delete unnecesary highsocres
 *   - intro
 *   - nicer menu screen
 *   - nicer grafics
 *   - more extras to high score (eg:
 *      maybe: really fast
 *      later: killed all enemies
 *      later maybe: took no damage
 *      found secret item)
 *   - maybe: make level chooser nicer
 *   - credits, create build dir (sort files, create INFOFILES)
 *   - maybe: saving point
 *   - more player states: stone, maybe fire (though not a permanent state,
 *          fueld by collectable coal), air (fueld by gas or something),
 *          maybe more, prob not
 *
 * * code
 *   - comments
 *   - refactoring (not now, but prob later. candidates: menu package,
 *          BlockMap, Player, all STATE files)
 *   - put ALL paths in GeneralNumbersAndFunctions,
 *      than rearange files and dirs in SideScroller
 *      (also delete or move old files)
 *
 * * performance increases (maybe needed later)
 *   - only render collectables (etc) when they are in shown map!
 *   - make one method for isAbove/Below/etc and return integer
 *          (blocks are loops through too often right now)
 *   - refactor/rewrite collision mechanism
 *   - replace float with int where reasonable
 *   - reduce string usage (especially reduce + operator)
 *   - do not render collidable image (transparent.png...) but find out how to
 *      disable rendering for it
 *   - maybe do not reload entire map, but only reset it...
 *   - only shoot and render canons when player is close
 *   - redo water. is on water block takes too much time
 *   - maybe sort collision/water/etc blocks according to x axis.
 *       then, not all have to be checked to be close to the player.
 *
 * Known bugs:
 *   - when a setting is added, settings file must be deleted,
 *          or game will crash
 *          (not confirmed)
 *   - when settings file does not exist, and settings are changed, changes are
 *          only applied after restart!
 *   - when in sticky mode, sometimes player cannot move left/right on sealing
 *
 * @author sam
 */
public class Game extends StateBasedGame {

    public static final int MAINMENUSTATEID = 0;
    public static final int GAMEPLAYSTATEID = 1;
    // public static final int INTROSTATE = 3;
    public static final int HIGHSCORESTATEID = 2;
    public static final int ERRORSTATEID = 5;
    public static final int LVLCHOOSERSTATEID = 6;
    public GameErrorState errorState;
    private static AppGameContainer app;

    /**
     * constructs new game.
     *
     * @param settings  settings
     */
    public Game(Settings settings) {
        super("The amazing Adventures of Stickman through the Galaxy "
                + "- in a Bubble!");
        errorState = new GameErrorState(ERRORSTATEID);
        this.addState(errorState);
        this.addState(new GameLevelChooserState(LVLCHOOSERSTATEID,
                "/data/levelchooser.tmx"));
        this.addState(new GameMenuState(MAINMENUSTATEID,
                settings.displayWidth / 2f, settings.displayHight / 2.5f, 20));
        GameHighScoreState gameHighScoreState
                = new GameHighScoreState(HIGHSCORESTATEID);
        this.addState(gameHighScoreState);
        this.addState(new GameState(GAMEPLAYSTATEID,
                GeneralNumbersAndFunctions.LEVELPATH + "lvl1The.Beginning.tmx",
                gameHighScoreState.getHighScore(), settings));
        this.enterState(MAINMENUSTATEID);
    }

    @Override
    public void initStatesList(
            GameContainer gameContainer) throws SlickException {
    }

    /**
     * start game.
     *
     * @param args args (no effect)
     * @throws SlickException  SlickException
     */
    public static void main(String[] args) throws SlickException {
        Settings settings = new Settings();
        app = new AppGameContainer(new ScalableGame(new Game(settings),
                settings.displayWidth, settings.displayHight,
                settings.fullscreen));
        //app.setClearEachFrame(false);
        app.setTargetFrameRate(250);
        if (Settings.debug) {
            app.setShowFPS(true);
        } else {
            app.setShowFPS(false);
        }
        try {
            app.setDisplayMode(settings.displayWidth, settings.displayHight,
                    settings.fullscreen);
            app.start();
        } catch (Exception e) {
            System.out.println("could not set " + settings.displayWidth + "x"
                    + settings.displayHight
                    + "\ntrying desktop display mode (may be distorted)");
            DisplayMode desktopDisplayMode = Display.getDesktopDisplayMode();
            app.setDisplayMode(desktopDisplayMode.getWidth(),
                    desktopDisplayMode.getHeight(),
                    desktopDisplayMode.isFullscreenCapable());
            app.start();
        }
    }
}
