/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sidescroller;

import java.awt.Font;
import java.io.File;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Input;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.UnicodeFont;
import org.newdawn.slick.state.BasicGameState;
import org.newdawn.slick.state.StateBasedGame;
import sidescroller.menu.AbstractMenuItem;
import sidescroller.menu.Menu;
import sidescroller.menu.MenuActionActivated;

/**
 * the HighScoreState displays the last high score as well as a collection
 * of previous scores.
 *
 * @author sam
 */
public class GameHighScoreState extends BasicGameState {

    private int stateID = -1;
    private HighScore highscore;
    private Menu chooselevel;
    private int chosenLevel;

    /**
     * possible states that can be rendered.
     */
    public enum STATES {
        /** lists levels, for which highscores are available. */
        LISTHIGHSCORES,
        /** shows the last aquired score. */
        SHOWRECENT,
        /**  shows highscores for one level (chosen in LISTHIGHSCORES state). */
        SHOWLVLSCORES,
        /** shows best time for one level(chosen in LISTHIGHSCORES state). */
        SHOWLVLTIME,
    }
    private STATES currentState;

    /**
     * constructs new GameHighScoreState.
     *
     * @param stateID stateID
     */
    public GameHighScoreState(int stateID) {
        super();
        this.stateID = stateID;
        highscore = new HighScore();
        currentState = STATES.SHOWRECENT;
    }

    /**
     * set state to be shown on enter state.
     *
     * @param state state
     */
    public void showState(STATES state) {
        currentState = state;
    }

    /**
     * returns high score.
     *
     * @return HighScore
     */
    public HighScore getHighScore() {
        return highscore;
    }

    @Override
    public int getID() {
        return stateID;
    }

    @Override
    public void enter(GameContainer container, StateBasedGame game)
            throws SlickException {
        super.enter(container, game);
    }

    @Override
    public void init(GameContainer container, StateBasedGame game)
            throws SlickException {
        initChooseLvlMenu(container);
    }

    @Override
    public void render(GameContainer container, StateBasedGame game, Graphics g)
            throws SlickException {
        switch (currentState) {
            case SHOWRECENT:
                int x = container.getWidth() / 2;
                int topY = 50;
                int yBetween = 20;
                g.drawString("Your Last Score: ", x, topY);
                topY += yBetween;
                topY += yBetween;

                g.drawString(highscore.getLastScore().toString(), x, topY);


                g.drawString("Place Score: " + highscore.getPlaceScore(), x,
                        container.getHeight() - 60);
                g.drawString("Place Time: " + highscore.getPlaceTime(), x,
                        container.getHeight() - 40);
                g.drawString("(press enter to continue)", x,
                        container.getHeight() - 20);

                break;

            case LISTHIGHSCORES:

                g.drawString("Choose Level for HighScore: ",
                        container.getWidth() / 2f, 10);
                chooselevel.render(g);
                break;
            case SHOWLVLSCORES:
                x = container.getWidth() / 2;

                g.drawString("Highscore Level " + chosenLevel, x, 10);

                g.drawString(highscore.getHighScoreFinal(chosenLevel), x, 50);

                g.drawString("(press enter to continue)", x,
                        container.getHeight() - 20);
                break;
            case SHOWLVLTIME:
                x = container.getWidth() / 2;

                g.drawString("Best Time Level " + chosenLevel, x, 10);

                g.drawString(highscore.getBestTime(chosenLevel), x, 50);

                g.drawString("(press enter to continue)", x,
                        container.getHeight() - 20);
                break;
            default:
                break;
        }

    }

    @Override
    public void update(GameContainer container, StateBasedGame game, int delta)
            throws SlickException {
        switch (currentState) {
            case SHOWRECENT:
                if (container.getInput().isKeyPressed(Input.KEY_ENTER)) {
                    game.enterState(Game.MAINMENUSTATEID);
                }
                break;
            case LISTHIGHSCORES:
                chooselevel.update(container, game, delta);
                break;
            case SHOWLVLSCORES:
                if (container.getInput().isKeyPressed(Input.KEY_ENTER)) {
                    currentState = STATES.SHOWLVLTIME;
                }
                break;
            case SHOWLVLTIME:
                if (container.getInput().isKeyPressed(Input.KEY_ENTER)) {
                    currentState = STATES.LISTHIGHSCORES;
                }
                break;
            default:
                break;
        }
    }

    /**
     * initiate choose level menu.
     *
     * @param container container
     */
    private void initChooseLvlMenu(GameContainer container) {
        UnicodeFont font = new UnicodeFont(
                new Font("arial", 2, 40), 40, true, true);

        // look up how many levels there are
        int amountLevels = 0;
        File dir = new File(GeneralNumbersAndFunctions.LEVELPATH);
        String[] children = dir.list();
        if (children != null) {
            for (int i = 0; i < children.length; i++) {
                // get name of file or directory
                String filename = children[i];
                if (filename.startsWith("lvl") && filename.endsWith(".tmx")) {
                    amountLevels++;
                }
            }
        }

        chooselevel = new Menu(container.getWidth() / 2f, 45f, 20f);

        // add items to level chooser
        String[] lvl = new String[amountLevels];
        for (int i = 0; i < lvl.length; i++) {
            lvl[i] = "Level " + (i + 1);
        }

        for (int i = 0; i < amountLevels; i++) {
            final int currI = i + 1;
            chooselevel.addItem(lvl[i],
                    font,
                    new MenuActionActivated() {

                        @Override
                        public void execute(GameContainer gc,
                                StateBasedGame sbg,
                                AbstractMenuItem item) throws SlickException {
                            chosenLevel = currI;
                            currentState = STATES.SHOWLVLSCORES;
                        }
                    });
        }
        chooselevel.addItem("back",
                font,
                new MenuActionActivated() {

                    @Override
                    public void execute(GameContainer gc, StateBasedGame sbg,
                            AbstractMenuItem item) throws SlickException {
                        sbg.enterState(Game.MAINMENUSTATEID);
                    }
                });
    }
}
