/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sidescroller;

import java.awt.Font;
import org.newdawn.slick.Color;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.UnicodeFont;
import sidescroller.menu.AbstractMenuItem;
import sidescroller.menu.Menu;
import sidescroller.menu.MenuActionActivated;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Input;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.font.effects.ColorEffect;
import org.newdawn.slick.gui.TextField;
import org.newdawn.slick.state.BasicGameState;
import org.newdawn.slick.state.StateBasedGame;
import sidescroller.menu.MenuAction;
import sidescroller.menu.MenuItemString;

/**
 *
 * @author sam
 */
public class GameMenuState extends BasicGameState {

    private Menu main;
    private Menu options;
    private Menu game;
    private Menu video;
    private Menu sound;
    private Menu input;
    private Menu chooselevel;
    private int stateID = -1;
    private UnicodeFont unicodeFont;
    private boolean couldLoadFont = false;
    private static final String BACK = "Back";
    private static final String HELP = "Welcome!\n"
            + "\nGeneral Controlls:"
            + "\npress left and right arrow keys to move."
            + "\npress 'a' to switch to jumpy mode and"
            + "\npress 's' to switch to sticky mode."
            + "\npress 'd' to switch to stoney mode."
            + "\n\nSticky mode Controlls:"
            + "\npress up to jump and"
            + "\nif sticking on sealing press down to fall."
            + "\n\nModes:"
            + "\nJumpy:"
            + "\n  * can jump the highest"
            + "\nSticky:"
            + "\n  * can stick to most stones. "
            + "\n  * tipp: because of its slow movement, this mode is good"
            + "\n    to move more secure under water."
            + "\nStoney:"
            + "\nis the fastest"
            + "\ncannot be killed by canon balls"
            + "\n available from level 4 on"
            + "\n\nPress Enter to return to main menu.";

    /**
     * list of menus which may be displayed.
     */
    private enum STATES {

        MAINMENU, OPTIONS, VIDEO, SOUND, INPUT,
        HELP, GAME, CHOOSELEVEL, CHANGENAME
    }
    private STATES currentState;
    private float centerXPosition;
    private float topYPosition;
    private float ySpaceBetweenItems;
    private SettingsLevel settingsLevel;
    private Settings settings;

    private TextField nameInput;

    /**
     * constructs new GameMenuState.
     *
     * @param stateID stateID
     * @param centerXPosition x position the center of this
     * menu should be rendered at
     * @param topYPosition y position the top of this
     * menu should be rendered at
     * @param ySpaceBetweenItems space between MenuItems on y axis
     */
    public GameMenuState(int stateID, float centerXPosition, float topYPosition,
            float ySpaceBetweenItems) {
        super();
        this.stateID = stateID;
        currentState = STATES.MAINMENU;
        this.centerXPosition = centerXPosition;
        this.topYPosition = topYPosition;
        this.ySpaceBetweenItems = ySpaceBetweenItems;
        settings = new Settings();
        settingsLevel = new SettingsLevel(settings);


    }

    @Override
    public int getID() {
        return this.stateID;
    }

    private void initMainMenu() {
        // <editor-fold defaultstate="collapsed" desc="initMainMenu">
        main = new Menu(centerXPosition, topYPosition, ySpaceBetweenItems);
        main.addItem("Start",
                unicodeFont,
                new MenuActionActivated() {

                    @Override
                    public void execute(GameContainer gc, StateBasedGame sbg,
                            AbstractMenuItem item) throws SlickException {
                        currentState = STATES.CHOOSELEVEL;
                    }
                });

        main.addItem("Options",
                unicodeFont,
                new MenuActionActivated() {

                    @Override
                    public void execute(GameContainer gc, StateBasedGame sbg,
                            AbstractMenuItem item) throws SlickException {
                        currentState = STATES.OPTIONS;
                    }
                });

        main.addItem("Highscore",
                unicodeFont,
                new MenuActionActivated() {

                    @Override
                    public void execute(GameContainer gc, StateBasedGame sbg,
                            AbstractMenuItem item) throws SlickException {
                        ((GameHighScoreState) sbg.getState(Game.HIGHSCORESTATEID)).showState(GameHighScoreState.STATES.LISTHIGHSCORES);
                        sbg.enterState(Game.HIGHSCORESTATEID);
                    }
                });

        main.addItem("Help",
                unicodeFont,
                new MenuActionActivated() {

                    @Override
                    public void execute(GameContainer gc, StateBasedGame sbg,
                            AbstractMenuItem item) throws SlickException {
                        currentState = STATES.HELP;
                    }
                });

        main.addItem("Exit",
                unicodeFont,
                new MenuActionActivated() {

                    @Override
                    public void execute(GameContainer gc, StateBasedGame sbg,
                            AbstractMenuItem item) throws SlickException {
                        gc.exit();
                    }
                });
        // </editor-fold>
    }

    private void initOptionsMenu() {
        // <editor-fold defaultstate="collapsed" desc="initOptionsMenu">
        options = new Menu(centerXPosition, topYPosition, ySpaceBetweenItems);
        options.addItem("Sound",
                unicodeFont,
                new MenuActionActivated() {

                    @Override
                    public void execute(GameContainer gc, StateBasedGame sbg,
                            AbstractMenuItem item) throws SlickException {
                        currentState = STATES.SOUND;
                    }
                });

        options.addItem("Video",
                unicodeFont,
                new MenuActionActivated() {

                    @Override
                    public void execute(GameContainer gc, StateBasedGame sbg,
                            AbstractMenuItem item) throws SlickException {
                        currentState = STATES.VIDEO;
                    }
                });

        options.addItem("Game",
                unicodeFont,
                new MenuActionActivated() {

                    @Override
                    public void execute(GameContainer gc, StateBasedGame sbg,
                            AbstractMenuItem item) throws SlickException {
                        currentState = STATES.GAME;
                    }
                });

        options.addItem(BACK,
                unicodeFont,
                new MenuActionActivated() {

                    @Override
                    public void execute(GameContainer gc, StateBasedGame sbg,
                            AbstractMenuItem item) throws SlickException {
                        currentState = STATES.MAINMENU;
                    }
                });
        // </editor-fold>
    }

    private void initGameMenu() {
        // <editor-fold defaultstate="collapsed" desc="initGameMenu">
        game = new Menu(centerXPosition, topYPosition, ySpaceBetweenItems);
        game.addItem("Input",
                unicodeFont,
                new MenuActionActivated() {

                    @Override
                    public void execute(GameContainer gc, StateBasedGame sbg,
                            AbstractMenuItem item) throws SlickException {
                        currentState = STATES.INPUT;
                    }
                });

        game.addItem("Change Name",
                unicodeFont,
                new MenuActionActivated() {

                    @Override
                    public void execute(GameContainer gc, StateBasedGame sbg,
                            AbstractMenuItem item) throws SlickException {

                        final UnicodeFont unicodeFont
                                = new UnicodeFont(new Font("Serif",
                                        Font.PLAIN, 20));
                        unicodeFont.addAsciiGlyphs();
                        unicodeFont.addGlyphs("1234567890,.?-+<>");
                        unicodeFont.getEffects().add(new ColorEffect());
                        try {
                            unicodeFont.loadGlyphs();
                        } catch (SlickException ex) {
                            GeneralNumbersAndFunctions.logError(
                                    "could not load unicodeFont!" + ex);
                        }

                        nameInput = new TextField(gc,
                                unicodeFont, 50, 50, 150, 30);
                        nameInput.setBackgroundColor(Color.lightGray);
                        nameInput.setBorderColor(
                                gc.getGraphics().getBackground());
                        nameInput.setFocus(true);
                        currentState = STATES.CHANGENAME;
                    }
                });

        String logDeathChosen;
        if (settings.logDeath) {
            logDeathChosen = "Log Death: True";
        } else {
            logDeathChosen = "Log Death: False";
        }

        game.addItem(logDeathChosen,
                unicodeFont,
                new MenuActionActivated() {

                    String dontLog = "Log Death: False";
                    String log = "Log Death: True";
                    int selected = 0;

                    @Override
                    public void execute(GameContainer gc, StateBasedGame sbg,
                            AbstractMenuItem item) throws SlickException {
                        if (selected == 0) {
                            selected = 1;
                            ((MenuItemString) item).setString(dontLog);
                            settings.setLogDeath(false);
                        } else {
                            selected = 0;
                            ((MenuItemString) item).setString(log);
                            settings.setLogDeath(true);
                        }
                    }
                });

        String logPositionChosen;
        if (settings.logDeath) {
            logPositionChosen = "Log Position: True";
        } else {
            logPositionChosen = "Log Position: False";
        }

        game.addItem(logPositionChosen,
                unicodeFont,
                new MenuActionActivated() {

                    String dontLog = "Log Position: False";
                    String log = "Log Position: True";
                    int selected = 0;

                    @Override
                    public void execute(GameContainer gc, StateBasedGame sbg,
                            AbstractMenuItem item) throws SlickException {
                        if (selected == 0) {
                            selected = 1;
                            ((MenuItemString) item).setString(dontLog);
                            settings.setLogPosition(false);
                        } else {
                            selected = 0;
                            ((MenuItemString) item).setString(log);
                            settings.setLogPosition(true);
                        }
                    }
                });

        String logLvlCompleteChosen;
        if (settings.logDeath) {
            logLvlCompleteChosen = "Log Lvl Complete: True";
        } else {
            logLvlCompleteChosen = "Log Lvl Complete: False";
        }

        game.addItem(logLvlCompleteChosen,
                unicodeFont,
                new MenuActionActivated() {

                    String dontLog = "Log Lvl Complete: False";
                    String log = "Log Lvl Complete: True";
                    int selected = 0;

                    @Override
                    public void execute(GameContainer gc, StateBasedGame sbg,
                            AbstractMenuItem item) throws SlickException {
                        if (selected == 0) {
                            selected = 1;
                            ((MenuItemString) item).setString(dontLog);
                            settings.setLogLvlComplete(false);
                        } else {
                            selected = 0;
                            ((MenuItemString) item).setString(log);
                            settings.setLogLvlComplete(true);
                        }
                    }
                });

        game.addItem(BACK,
                unicodeFont,
                new MenuActionActivated() {

                    @Override
                    public void execute(GameContainer gc, StateBasedGame sbg,
                            AbstractMenuItem item) throws SlickException {
                        currentState = STATES.OPTIONS;
                    }
                });
        // </editor-fold>
    }

    private void initVideoMenu() {
        // <editor-fold defaultstate="collapsed" desc="initOptionsMenu">
        video = new Menu(centerXPosition, topYPosition, ySpaceBetweenItems);
        video.addItem(BACK,
                unicodeFont,
                new MenuActionActivated() {

                    @Override
                    public void execute(GameContainer gc, StateBasedGame sbg,
                            AbstractMenuItem item) throws SlickException {
                        currentState = STATES.OPTIONS;
                    }
                });
        // </editor-fold>
    }

    private void initSoundMenu() {
        // <editor-fold defaultstate="collapsed" desc="initOptionsMenu">
        sound = new Menu(centerXPosition, topYPosition, ySpaceBetweenItems);
        sound.addItem("Off",
                unicodeFont,
                new MenuAction() {

                    @Override
                    public void execute(GameContainer gc, StateBasedGame sbg,
                            AbstractMenuItem item) throws SlickException {
                    }

                    @Override
                    public boolean isActivated() {
                        return false;
                    }
                });

        sound.addItem(BACK,
                unicodeFont,
                new MenuActionActivated() {

                    @Override
                    public void execute(GameContainer gc, StateBasedGame sbg,
                            AbstractMenuItem item) throws SlickException {
                        currentState = STATES.OPTIONS;
                    }
                });
        // </editor-fold>
    }

    private void initInputMenu() {
        // <editor-fold defaultstate="collapsed" desc="initOptionsMenu">
        input = new Menu(centerXPosition, topYPosition, ySpaceBetweenItems);
        input.addItem(BACK,
                unicodeFont,
                new MenuActionActivated() {

                    @Override
                    public void execute(GameContainer gc, StateBasedGame sbg,
                            AbstractMenuItem item) throws SlickException {
                        currentState = STATES.GAME;
                    }
                });
        // </editor-fold>
    }

    private void initChooseLvlMenu(StateBasedGame sbg) {
        // <editor-fold defaultstate="collapsed" desc="initOptionsMenu">
        chooselevel = new Menu(centerXPosition, 10, ySpaceBetweenItems);
        ((GameState) sbg.getState(Game.GAMEPLAYSTATEID)).setSettingsLevel(settingsLevel);

        // add items to level chooser
        String[] lvl = new String[settingsLevel.getNumberOfLevels()];
        for (int i = 0; i < lvl.length; i++) {
            lvl[i] = "Level " + (i + 1) + " - "
                    + settingsLevel.getLevelDisplayName(i + 1);
        }

        for (int i = 0; i < settingsLevel.getNumberOfLevels(); i++) {
            final int currI = i + 1;
            chooselevel.addItem(lvl[i],
                    unicodeFont,
                    new MenuAction() {

                        @Override
                        public void execute(GameContainer gc, StateBasedGame sbg,
                                AbstractMenuItem item) throws SlickException {
                            ((GameState) sbg.getState(Game.GAMEPLAYSTATEID)).setLevel(GeneralNumbersAndFunctions.LEVELPATH + settingsLevel.getLevelName(currI));
                            ((GameState) sbg.getState(Game.GAMEPLAYSTATEID)).setLevel(currI);
                            //gc.reinit();
                            ((GameState) sbg.getState(Game.GAMEPLAYSTATEID)).init(gc, sbg);
                            sbg.enterState(Game.GAMEPLAYSTATEID);
                        }

                        @Override
                        public boolean isActivated() {
                            return settingsLevel.isLevelOpen(currI);
                        }
                    });
        }
        chooselevel.addItem(BACK,
                unicodeFont,
                new MenuActionActivated() {

                    @Override
                    public void execute(GameContainer gc, StateBasedGame sbg,
                            AbstractMenuItem item) throws SlickException {
                        currentState = STATES.MAINMENU;
                    }
                });
        // </editor-fold>
    }

    @Override
    public void init(GameContainer gc, StateBasedGame sbg)
            throws SlickException {

        try {
            unicodeFont = new UnicodeFont(new Font("Serif", Font.PLAIN, 9));
            unicodeFont.addAsciiGlyphs();
            unicodeFont.addGlyphs("1234567890,.?-+<>");
            unicodeFont.getEffects().add(new ColorEffect());
            unicodeFont.loadGlyphs();
            couldLoadFont = true;
        } catch (Exception e) {
            couldLoadFont = false;
            GeneralNumbersAndFunctions.logError("could not load font. " + e);
        }

        initMainMenu();
        initOptionsMenu();
        initGameMenu();
        initVideoMenu();
        initSoundMenu();
        initInputMenu();
        initChooseLvlMenu(sbg);

    }

    @Override
    public void render(GameContainer container, StateBasedGame sbg, Graphics g)
            throws SlickException {

        switch (currentState) {
            case MAINMENU:
                main.render(g);
                break;
            case OPTIONS:
                options.render(g);
                break;
            case VIDEO:
                video.render(g);
                break;
            case SOUND:
                sound.render(g);
                break;
            case INPUT:
                input.render(g);
                break;
            case HELP:
                if (couldLoadFont) {
                    unicodeFont.drawString(5F, 5F, HELP, Color.white);
                } else {
                    g.drawString(HELP, 5, 5);
                }
                break;
            case CHOOSELEVEL:
                chooselevel.render(g);
                break;
            case GAME:
                game.render(g);
                break;
            case CHANGENAME:
                    g.drawString("Enter New Name:", 50, 20);
                  nameInput.render(container, g);
                break;
            default:
                break;
        }
    }

    @Override
    public void update(GameContainer gc, StateBasedGame sbg, int i)
            throws SlickException {

        switch (currentState) {
            case MAINMENU:
                main.update(gc, sbg, i);
                break;
            case OPTIONS:
                options.update(gc, sbg, i);
                break;
            case VIDEO:
                video.update(gc, sbg, i);
                break;
            case SOUND:
                sound.update(gc, sbg, i);
                break;
            case INPUT:
                input.update(gc, sbg, i);
                break;
            case HELP:
                Input userInput = gc.getInput();
                if (userInput.isKeyPressed(Input.KEY_ENTER)) {
                    currentState = STATES.MAINMENU;
                }
                break;
            case CHOOSELEVEL:
                chooselevel.update(gc, sbg, i);
                break;
            case GAME:
                game.update(gc, sbg, i);
                break;
            case CHANGENAME:
                    if (gc.getInput().isKeyPressed(Input.KEY_ENTER)) {
                        nameInput.deactivate();
                        settings.setPlayerName(nameInput.getText());
                        currentState = STATES.GAME;
                    }
                break;
            default:
                break;
        }
    }
}
