/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sidescroller;

import java.util.ArrayList;
import java.util.List;
import org.newdawn.slick.Color;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.Input;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.state.BasicGameState;
import org.newdawn.slick.state.StateBasedGame;

/**
 * test side scroller.
 *
 * @author sam
 */
public class GameState extends BasicGameState {

    private float time;
    private String level;
    private int stateID = -1;
    private Camera camera;
    private BlockMap map;
    private Player player;
    // TODO these fields + logic still need work
    // they do work as is, but
    // - background will break if the container size is changed,
    // - updates does not represent seconds
    //      (though it is close and also independend of game speed)
    // - gear also works as is, but it is not nicely writen/new Collectables
    //      will be hard to implement (maybe...)
    private Image background;
    private int updates;
    private List<Collectable> gear;
    private List<Canon> canons;
    // TODO think about moving them back to Canon class completely
    private List<CanonBall> canonBalls;
    private HighScore highscore;
    private SettingsLevel settingsLevel;
    private int lvlPlaying = -1;
    private Settings settings;
    private boolean drawBG;

    /**
     * constructs new game state.
     *
     * @param stateID stateID
     * @param lvl level to construct game state for
     * @param highscore highscore
     * @param settings settings
     */
    public GameState(int stateID, String lvl,
            HighScore highscore, Settings settings) {
        super();
        this.stateID = stateID;
        this.level = lvl;
        this.highscore = highscore;
        this.settings = settings;
    }

    @Override
    public int getID() {
        return this.stateID;
    }

    /**
     * sets the level which should be loaded on state ender.
     *
     * @param lvlPlaying lvlPlaying
     */
    public void setLevel(int lvlPlaying) {
        this.lvlPlaying = lvlPlaying;
    }

    /**
     * sets settingsLevel.
     *
     * @param settingsLevel settingsLevel
     */
    public void setSettingsLevel(SettingsLevel settingsLevel) {
        this.settingsLevel = settingsLevel;
    }

    /**
     * sets level file.
     *
     * @param level path + name to level file
     */
    public void setLevel(String level) {
        this.level = level;
    }

    @Override
    public void init(GameContainer container, StateBasedGame game)
            throws SlickException {
        // FIXME it seems, that one cannot enter a different state in init...
        // now, an exception will be printed, and lvl1 will be loaded...
        time = 0;
        updates = 0;
        try {
            map = new BlockMap(level);
        } catch (Exception ex) {
            GeneralNumbersAndFunctions.logError(ex.getMessage());
            game.enterState(Game.ERRORSTATEID);
        }

        try {
            player = map.loadPlayer(settings.playerName);
            camera = new Camera(container, map.getTiledMap());
            gear = map.loadGear();
            canons = map.loadCanons();
        } catch (Exception ex) {
            GeneralNumbersAndFunctions.logError("GameState.init(): "
                    + "could not load player. " + ex);
            game.enterState(Game.ERRORSTATEID);
        }
        canonBalls = new ArrayList<CanonBall>();

        try {
            background = new Image(GeneralNumbersAndFunctions.LEVELPATH
                    + map.getBackgroundPic());
        } catch (Exception ex) {
            GeneralNumbersAndFunctions.logError(
                    "could not load background image: "
                    + GeneralNumbersAndFunctions.LEVELPATH
                    + map.getBackgroundPic());
            //background = new Image(new ImageBuffer(1, 1));
        }

        drawBG = !("transparent.png".equals(map.getBackgroundPic()))
                && background != null;

    }
    private int oldUpdates = 0;
    private int myUpdates = 0;

    @Override
    public void update(GameContainer container, StateBasedGame game, int delta)
            throws SlickException {

        if (settings.logPosition) {
            // log position approximately every second
            myUpdates = (int) (updates / 100f);
            if (myUpdates != oldUpdates) {
                GeneralNumbersAndFunctions.logPosition(level + ";"
                        + (int) player.position.x + " "
                        + (int) player.position.y);
                oldUpdates = myUpdates;
            }
        }

        time -= delta;

        if (time < 0) {
            updates++;
            time = GeneralNumbersAndFunctions.DELTANORMALIZATION;

            player.setIsInWater(map.isOnWaterBlock(player));

            // handle state change imput
            if (container.getInput().isKeyPressed(Input.KEY_A)) {
                player.changeState(Player.STATES.JUMPY);
            }
            if (container.getInput().isKeyPressed(Input.KEY_S)) {
                player.changeState(Player.STATES.STICKY);
            }
            if (container.getInput().isKeyPressed(Input.KEY_D)) {
                if (lvlPlaying >= 4) {
                    player.changeState(Player.STATES.STONEY);
                }
            }

            // handle movement input
            if (container.getInput().isKeyDown(Input.KEY_LEFT)) {
                player.moveLeft();
            }
            if (container.getInput().isKeyDown(Input.KEY_RIGHT)) {
                player.moveRight();
            }
            if (container.getInput().isKeyDown(Input.KEY_UP)) {
                player.initiateJump();
            }
            if (container.getInput().isKeyDown(Input.KEY_DOWN)) {
                player.drop();
            }

            player.checkJump();
            player.applyStatebasedMovement();

            ckeckCollected();

            camera.centerOn(player.getCollisionShape());

            ckeckCanons();

            if (!player.isAlive()) {
                playerDied();
                game.enterState(Game.MAINMENUSTATEID);
            }

            int lvlComplete = map.isLevelComplete(player);
            if (lvlComplete != 0) {
                levelComplete(lvlComplete);
                ((GameHighScoreState) game.getState(
                        Game.HIGHSCORESTATEID)).showState(
                        GameHighScoreState.STATES.SHOWRECENT);
                game.enterState(Game.HIGHSCORESTATEID);
            }
        }
    }

    /**
     * call when player died.
     * If logDeath is enabled, it will log the death.
     */
    private void playerDied() {
        if (settings.logDeath) {
            GeneralNumbersAndFunctions.logDeath(level + ";"
                    + (int) player.position.x + " "
                    + (int) player.position.y);
        }
    }

    /**
     * call when level is complete.
     *
     * @param lvlComplete lvlComplete code.
     */
    private void levelComplete(int lvlComplete) {
        if (settings.logLvlComplete) {
            GeneralNumbersAndFunctions.logLevelComplete(level + ";"
                    + ((int) (updates / 100f)));
        }
        settingsLevel.openLevel(lvlPlaying + 1);

        ArrayList<HighScoreBonus> bonuses = new ArrayList<HighScoreBonus>();
        if (lvlComplete == 2) {
            bonuses.add(new HighScoreBonus("Secret Exit Bonus", 150));
        }

        if (gear.isEmpty()) {
            bonuses.add(new HighScoreBonus("All Gear Bonus", 200));
        }

        int stateChanges = player.getStateChanges();
        if (stateChanges < 10) {
            bonuses.add(new HighScoreBonus(
                    "Below 10 StateChanges Bonus", 25));
        }
        if (stateChanges < 5) {
            bonuses.add(new HighScoreBonus(
                    "Below 5 StateChanges Bonus", 50));
        }
        if (stateChanges < 3) {
            bonuses.add(new HighScoreBonus(
                    "Below 3 StateChanges Bonus", 100));
        }
        if (stateChanges < 2) {
            bonuses.add(new HighScoreBonus(
                    "Below 2 StateChanges Bonus", 150));
        }

        highscore.addHighScore(player.getName(),
                (int) (updates / 100f), 200,
                player.getMoney(), bonuses, lvlPlaying);
    }

    /**
     * checks if player collected collectable.
     */
    private void ckeckCollected() {
        List remove = new ArrayList();
        for (Collectable collectable : gear) {
            // check first if player is close (performes best)
            if (collectable.isCloseTo(player.getPosition().x,
                    player.getPosition().y)) {
                // check if player is actually colliding (performs not so good)
                if (player.isCollidingWith(collectable)) {
                    player.collected(collectable);
                    remove.add(collectable);
                }
            }
        }
        gear.removeAll(remove);
    }

    /**
     * checks and updates canons and canon balls.
     */
    private void ckeckCanons() {
        // fire all canons
        // TODO do NOT fire all of them, only those that are close to the player.
        for (Canon canon : canons) {
            CanonBall fire = canon.fire();
            if (fire != null) {
                canonBalls.add(fire);
            }
        }

        // move all canonBalls.
        // TODO if canonBall moves far away from player, delete it.
        List remove = new ArrayList();
        for (CanonBall canonBall : canonBalls) {
            int move = canonBall.move(map, player);
            if (move == 2) {
                if (player.getCurrentState().equals(Player.STATES.STONEY)) {
                    remove.add(canonBall);
                } else {
                    player.kill();
                }
            } else if (move == 1) {
                remove.add(canonBall);
            }
        }
        canonBalls.removeAll(remove);
    }

    /**
     * render canons and canon balls.
     *
     * @param g graphics context
     */
    private void renderCanons(Graphics g) {
        for (Canon canon : canons) {
            // TODO only render when in camera view (performance)
            canon.render(g);
        }

        for (CanonBall canonBall : canonBalls) {
            canonBall.render(g);
        }
    }

    @Override
    public void render(GameContainer container, StateBasedGame game, Graphics g)
            throws SlickException {
        if (drawBG) {
            background.draw(0, 0);
        } else {
            g.setBackground(Color.black);
        }

        camera.drawMap(g, Integer.toString((int) (updates / 100f)),
                Integer.toString(player.getMoney()));
        camera.translateGraphics();

        player.render(g);

        for (Collectable collectable : gear) {
            // TODO only render when in camera view (performance)
            collectable.render(g);
        }

        renderCanons(g);

        if (Settings.debug) {
            map.renderCollisionShapes(g);
        }

        // TODO drawing the string here instead of in camera is a lot nicer.
        // but (un-)translate graphics must be called. performance?
        //camera.untranslateGraphics();
        // g.drawString("time: " + Integer.toString((int) (updates / 100f))
        // + "money: " + Integer.toString(player.getMoney()), 20, 20);
        //camera.translateGraphics();
    }
}
