/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sidescroller;

import java.io.File;
import java.io.FileWriter;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

/**
 * This class contains game settings which users should not be able to change
 * as well as static functions.
 *
 * @author sam
 */
public final class GeneralNumbersAndFunctions {

    /**
     * do not call this constructor.
     */
    private GeneralNumbersAndFunctions() {
        // prevents calls from subclass
        throw new UnsupportedOperationException();
    }

    /** amount of live the player is given at the start. */
    public static final int STARTLIVE = 100;
    /** approximation used to check if object is close to another (in pixel). */
    public static final int ISCLOSETOAPPROCIMATION = 50;
    /** minimal milliseconds to wait before parsing new input. */
    public static final float DELTANORMALIZATION = 7;
    /**
     * gravity will be added to <code>JUMPINONETURN</code> every turn
     * to simulate a physically correct jump.
     *
     * @see IJumpable
     */
    public static final float GRAVITY = 0.1f;
    /** starting distance to travel in one turn in a jump. */
    public static final float JUMPINONETURN = -2.95f;
    /** amount to moveDiagonally at a time. */
    public static final float SPEED = 1.5f;

    /** percentage to apply to JUMPINONETURN in sticky mode.  */
    public static final float STICKYSTARTJUMPPERCENT = 0.85f;
    /** percentage to apply to SPEED in sticky mode. */
    public static final float STICKYSPEEDPERCENT = 0.7f;
    /** percentage to apply to JUMPINONETURN in jumpy mode.  */
    public static final float JUMPYSTARTJUMPPERCENT = 1.2f;
    /** percentage to apply to SPEED in jumpy mode. */
    public static final float JUMPYPEEDPERCENT = 1.4f;

    /** percentage to apply to JUMPINONETURN in jumpy mode.  */
    public static final float STONEYSTARTJUMPPERCENT = 0.5f;
    /** percentage to apply to SPEED in jumpy mode. */
    public static final float STONEYSPEEDPERCENT = 2f;

    /** amount of scores to be saved per level. */
    public static final int SAVETOPNSCORES = 10;
    /** maximal name lenght that may be displayed. */
    public static final int MAXNAMELENGHT = 12;
    /** path to the level files. */
    public static final String LEVELPATH = System.getProperty("user.dir")
            + File.separator + "data" + File.separator;
    // TODO implement below (water resistance)
    public static final float WATERSPEEDPERCENT = 0.5f;
    public static final float WATERSTARTJUMPPERCENT = 0.5f;
    public static final boolean ISLOGPOSITION = true;
    public static final boolean ISLOGDEATH = true;
    public static final boolean ISLOGLVLCOMPLETE = true;

    /**
     * prints a point to sto surrounded with parentheses.
     *
     * @param x x
     * @param y y
     */
    public static void printPoint(float x, float y) {
        System.out.print("(" + x + ", " + y + ")");
    }

    /**
     * logs and prints an error.
     *
     * @param s error message
     */
    public static void logError(String s) {
        System.out.println(s);
        log(s, "/logs/errorLog");
    }

    /**
     * logs the death of a player.
     * Position as well as level must be passed, seperated by a semicolon.
     * x- and y-Position must be seperated by a space.
     *
     * @param s "level";"xPos" "yPos"
     */
    public static void logDeath(String s) {
        log(cleanLogMessage(s), "/logs/deathLog");
    }

    /**
     * logs the position of a player.
     * Position as well as level must be passed, seperated by a semicolon.
     * x- and y-Position must be seperated by a space.
     *
     * @param s "level";"xPos" "yPos"
     */
    public static void logPosition(String s) {
        // TODO maybe do not log position after certain time but after
        // space traveled. currently, the heatmap is misleading.
        // it does NOT really display where players go, but where
        // they take the longest
        log(cleanLogMessage(s), "/logs/positionLog");
    }

    /**
     * logs when a level is complete.
     *
     * @param s "level";"time"
     */
    public static void logLevelComplete(String s) {
        log(cleanLogMessage(s), "/logs/lvlCompleteLog");
    }

    /**
     * removes path from lvl name.
     * Message should have format:
     * <p>
     * "level";"additionalStuff"
     *
     * @param message message
     * @return cleaned message.
     */
    private static String cleanLogMessage(String message) {
        // delete path from lvl name
        String[] lvlAndPoint = message.split(";");
        String[] lvl = lvlAndPoint[0].split("/");
        String lvlName = lvl[lvl.length - 1];
        return lvlName + ";" + lvlAndPoint[1];
    }

    /**
     * appends the given string to a log file and adds a newline character.
     * The current date and time will be prefixed and seperated by a semicolon.
     *
     * @param s log
     * @param logName file name
     */
    public static void log(String s, String logName) {
        String path = System.getProperty("user.dir")
                + System.getProperty("file.separator");
        String preString = getDate("yyyy.MM.dd-HH:mm:ss-z");

        try {
            FileWriter fw = new FileWriter(path + logName, true);
            fw.write(preString + ";" + s + "\n");
            fw.close();
        } catch (Exception ex) {
            System.out.println("log(): ERROR writing to log:"
                    + path + logName + ": " + ex);
        }
    }

    /**
     * returns the current date in desired format.
     * <p>
     * example for format: "yyyy.MM.dd-HH:mm:ss-z" or "yyyy.mm.dd".
     *
     * @param format format
     * @return date
     */
    public static String getDate(String format) {
        DateFormat dateFormat = new SimpleDateFormat(format, Locale.US);
        return dateFormat.format(new Date());
    }
}
