/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sidescroller;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Comparator;

/**
 * // TODO refactor this (or rewrite it completely). Levels are not considerd,
 *      calculation is only saved for last score, etc
 * @author sam
 */
public class HighScore implements Serializable {

    private static final long serialVersionUID = 1L;
    /**
     * The first list holds lists of single scores.
     * Entry 0 holds single scores for lvl 1, and so on.
     */
    private ArrayList<ArrayList<HighScoreSingleScore>> highscores;

    /* detailed score of last calculated score: */
    private HighScoreSingleScore lastScore;
    private int placeScore;
    private int placeTime;
    private String highScoreFilePath = System.getProperty("user.dir")
            + File.separator
            + "highscore";

    /**
     * constructs new HighScore.
     * Old highscores are loaded from highScoreFilePath.
     *
     */
    public HighScore() {
        readHighScoreFile(highScoreFilePath);
        lastScore = new HighScoreSingleScore("", 0, 0, 0,
                new ArrayList<HighScoreBonus>(), 0);
    }

    /**
     * returns a string containing the time scores for the given level.
     * The string will contain the players name and their time score.
     *
     * @param level level
     * @return list of best times for level
     */
    public String getBestTime(int level) {
        return getHighScore(level, new HighScoreType() {

            @Override
            public int getValue(HighScoreSingleScore singleScore) {
                return singleScore.getTimeNeeded();
            }
        },
                new SingleScoreComparatorTime());
    }

    /**
     * returns a string containing the final high scores for the given level.
     * The string will contain the players name and their final score.
     *
     * @param level level
     * @return list of best scores for level
     */
    public String getHighScoreFinal(int level) {
        return getHighScore(level, new HighScoreType() {

            @Override
            public int getValue(HighScoreSingleScore singleScore) {
                return singleScore.getFinalScore();
            }
        },
                new SingleScoreComparatorFinal());
    }

    /**
     * returns a string containing the scores for the given level.
     * The string will contain the players name and value defined
     * by HighScoreType.
     *
     * @param level level
     * @param type which values are to get?
     * @param comparator used to sort list
     * @return list of scores for level
     */
    private String getHighScore(int level, HighScoreType type,
            Comparator<HighScoreSingleScore> comparator) {
        ArrayList<HighScoreSingleScore> highscore = null;
        try {
            highscore = highscores.get(level - 1);
        } catch (Exception e) {
            return "no such level exists or no score aquired yet.";
        }
        java.util.Collections.sort(highscore, comparator);

        StringBuffer buf = new StringBuffer();
        for (int i = 0; (i < highscore.size())
                && i < GeneralNumbersAndFunctions.SAVETOPNSCORES; i++) {
            HighScoreSingleScore singleScore = highscore.get(i);
            buf.append((i + 1) + ". " + singleScore.getName()
                    + getSpaces(singleScore.getName())
                    + ": " + type.getValue(singleScore) + "\n");

        }
        return buf.toString();
    }

    /**
     * interface which can be used to decide which value should be retrieved
     * from HighScoreSingleScore.
     */
    private interface HighScoreType {

        /**
         * returns a value from HighScoreSingleScore.
         *
         * @param singleScore singleScore
         * @return value
         */
        int getValue(HighScoreSingleScore singleScore);
    }

    /**
     * return amount of spaces to aline name.
     *
     * @param name username
     * @return string containing spaces
     */
    private String getSpaces(String name) {
        int amount = (GeneralNumbersAndFunctions.MAXNAMELENGHT - name.length())
                + 4;
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < amount; i++) {
            buf.append(' ');
        }
        return buf.toString();
    }

    /**
     * loads old highscore from file.
     *
     * @param path file + path to highscore file
     */
    private void readHighScoreFile(String path) {

        File highScoreFile = new File(highScoreFilePath);
        if (!highScoreFile.exists()) {
            // debug
            highscores = new ArrayList<ArrayList<HighScoreSingleScore>>();
            return;
        }

        try {
            FileInputStream fis =
                    new FileInputStream(path);
            ObjectInputStream ois = new ObjectInputStream(fis);
            highscores = (ArrayList<ArrayList<HighScoreSingleScore>>) ois.readObject();
        } catch (Exception ioe) {
            GeneralNumbersAndFunctions.logError(
                    this.getClass().getCanonicalName()
                    + ".readHighScoreFile(): could not read "
                    + highScoreFilePath + ": " + ioe);
        }


    }

    /**
     * adds a new highscore.
     *
     * @param name player name
     * @param time time it took to complete level
     * @param expectedTime time expected to complete level
     * @param money money collected in level
     * @param bonuses bonuses collected in level
     * @param level level
     */
    public void addHighScore(String name, int time, int expectedTime,
            int money, ArrayList<HighScoreBonus> bonuses, int level) {

        String adjustedName = "";
        if (name.length() > GeneralNumbersAndFunctions.MAXNAMELENGHT - 1) {
            adjustedName = name.substring(0, GeneralNumbersAndFunctions.MAXNAMELENGHT);
        }

        // TODO replace hack with good code.
        // what happens: if eg lvl 1 has no score
        // and someone reaches end of lvl 2,
        // game would crash without this.
        if (level - 1 >= highscores.size()) {
            for (int i = 0; i <= level - 1; i++) {
                try {
                    highscores.get(i);
                } catch (Exception e) {
                    highscores.add(new ArrayList<HighScoreSingleScore>());
                }
            }
        }
        ArrayList<HighScoreSingleScore> highScore = highscores.get(level - 1);

        // create new single score
        HighScoreSingleScore highScoreSingleScore
                = new HighScoreSingleScore(adjustedName, time, expectedTime,
                money, bonuses, level);
        lastScore = highScoreSingleScore;

        // add new single score
        highScore.add(highScoreSingleScore);

        deleteUnneccesaryEntries(highScore);

        // sort by final score
        java.util.Collections.sort(highScore, new SingleScoreComparatorFinal());
        placeScore = highScore.indexOf(highScoreSingleScore) + 1;

        java.util.Collections.sort(highScore, new SingleScoreComparatorTime());
        placeTime = highScore.indexOf(highScoreSingleScore) + 1;

        saveHighScoreFile();
    }

    /**
     * deletes single score when it does not appear in top 10 time or score
     * lists.
     *
     * @param highScore highScore
     */
    private void deleteUnneccesaryEntries(
            ArrayList<HighScoreSingleScore> highScore) {
        // TODO implement deletion of unneccesary highscore entries
//        // if list is bigger than it should be, remember last Entries
//        // and delete them in case they do not hold top 10 time record.
//        if (highScore.size() >= GeneralNumbersAndFunctions.SAVETOPNSCORES) {
//            ArrayList<HighScoreSingleScore> maybeRemove
//                    = new ArrayList<HighScoreSingleScore>();
//            for (int i = GeneralNumbersAndFunctions.SAVETOPNSCORES + 1;
//                        i < highScore.size(); i++) {
//                maybeRemove.add(highScore.get(i));
//            }
//        }
//        // TODO only delete, if it does not hold time record!
//        if (highScore.size() == GeneralNumbersAndFunctions.SAVETOPNSCORES) {
//            highScore.remove(GeneralNumbersAndFunctions.SAVETOPNSCORES - 1);
//        }
    }
    private boolean triedOnce = false;

    /**
     * saves high scores.
     */
    private void saveHighScoreFile() {
        try {
            FileOutputStream fis =
                    new FileOutputStream(highScoreFilePath);
            ObjectOutputStream ois = new ObjectOutputStream(fis);
            ois.writeObject(highscores);
        } catch (Exception e) {
            File highScoreFile = new File(highScoreFilePath);
            if (!highScoreFile.exists()) {
                // file did not exist. try to create it
                try {
                    highScoreFile.createNewFile();
                } catch (IOException ex) {
                    GeneralNumbersAndFunctions.logError(
                            this.getClass().getCanonicalName()
                            + ".saveHighScoreFile(): could not create "
                            + highScoreFilePath + ": " + ex);
                    if (!triedOnce) {
                        saveHighScoreFile();
                        triedOnce = true;
                    }
                }
            } else {
                // file did exist, different error.
                GeneralNumbersAndFunctions.logError(
                        this.getClass().getCanonicalName()
                        + ".saveHighScoreFile() : " + e);
            }
        }
    }

    /**
     * returns the latest gained single score.
     *
     * @return lastScore
     */
    public HighScoreSingleScore getLastScore() {
        return lastScore;
    }

    /**
     * returns the place of the last gained score.
     *
     * @return placeScore
     */
    public int getPlaceScore() {
        return placeScore;
    }

    /**
     * returns the place of the last gained time.
     *
     * @return placeTime
     */
    public int getPlaceTime() {
        return placeTime;
    }

    /**
     * comparator to use to sort HighScoreSingleScore according to final score.
     */
    protected static class SingleScoreComparatorFinal
            implements Comparator<HighScoreSingleScore>, Serializable {

        private static final long serialVersionUID = 1L;

        @Override
        public int compare(HighScoreSingleScore a, HighScoreSingleScore b) {
            if (a.getFinalScore() > b.getFinalScore()) {
                return -1;
            } else if (a.getFinalScore() < b.getFinalScore()) {
                return 1;
            }
            return 0;
        }
    }

    /**
     * comparator to use to sort HighScoreSingleScore according to time.
     */
    protected static class SingleScoreComparatorTime
            implements Comparator<HighScoreSingleScore>, Serializable {

        private static final long serialVersionUID = 1L;

        @Override
        public int compare(HighScoreSingleScore a, HighScoreSingleScore b) {
            if (a.getFinalScore() < b.getFinalScore()) {
                return -1;
            } else if (a.getTimeNeeded() > b.getTimeNeeded()) {
                return 1;
            }
            return 0;
        }
    }
}
