/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sidescroller;

import java.io.Serializable;
import java.util.List;

/**
 * This class stores a single score.
 * <p>
 * A single score is the score a player gets when finishing a level.
 *
 * @author sam
 */
public class HighScoreSingleScore implements Serializable {

    private static final long serialVersionUID = 1L;
    private int timeNeeded;
    private int timeExpected;
    private int moeny;
    private int finalScore;
    private List<HighScoreBonus> bonuses;
    private int totalBonuses;
    private int level;
    private String name;

    /**
     * constructs new single score.
     *
     * @param name name of player
     * @param time time to complete lvl
     * @param expectedTime expected time to complete lvl
     * @param money money collected in lvl
     * @param bonuses bonuses aquired in lvl
     * @param level level
     */
    public HighScoreSingleScore(String name, int time, int expectedTime,
            int money, List<HighScoreBonus> bonuses, int level) {
        this.finalScore = calculateScore(time, expectedTime, money, bonuses);
        this.name = name;
        this.moeny = money;
        this.timeExpected = expectedTime;
        this.timeNeeded = time;
        this.bonuses = bonuses;
        this.level = level;
    }

    /**
     * returns the total score.
     *
     * @return finalScore
     */
    public int getFinalScore() {
        return finalScore;
    }

    /**
     * returns the money collected.
     *
     * @return moeny
     */
    public int getMoeny() {
        return moeny;
    }

    /**
     * returns the time
     * expected for the level this single score was achieved in.
     *
     * @return timeExpected
     */
    public int getTimeExpected() {
        return timeExpected;
    }

    /**
     * returns the time needed to complete the level.
     *
     * @return timeNeeded
     */
    public int getTimeNeeded() {
        return timeNeeded;
    }

    /**
     * returns the name of the player.
     *
     * @return name
     */
    public String getName() {
        return name;
    }

    /**
     * returns the level in which this score was achieved.
     *
     * @return level
     */
    public int getLevel() {
        return level;
    }

    /**
     * returns the added amount of bonuses collected in the level.
     *
     * @return totalBonuses
     */
    public int getTotalBonuses() {
        return totalBonuses;
    }

    /**
     * calculates score.
     *
     * @param time time
     * @param expectedTime expectedTime
     * @param money money
     * @param bonuses bonuses
     * @return total score
     */
    private int calculateScore(int time, int expectedTime, int money,
            List<HighScoreBonus> bonuses) {
        // calculate
        int finalScore = 0;

        for (HighScoreBonus highScoreBonus : bonuses) {
            finalScore += highScoreBonus.getValue();
        }
        totalBonuses = finalScore;

        finalScore += (expectedTime - time);

        finalScore += money;

        return finalScore;
    }

    @Override
    public String toString() {
        StringBuffer buf = new StringBuffer(70);

        buf.append("Money:       ");
        buf.append(moeny);
        buf.append("\nTime (");
        buf.append(timeNeeded);
        buf.append("):    ");
        buf.append((timeExpected - timeNeeded));
        buf.append("\n");

        for (HighScoreBonus highScoreBonus : bonuses) {
            buf.append(highScoreBonus.getName() + " : "
                    + highScoreBonus.getValue() + "\n");
        }

        buf.append("final score : ");
        buf.append(finalScore);
        return buf.toString();
    }
}
