/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sidescroller;

import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.geom.Vector2f;
import org.newdawn.slick.state.StateBasedGame;

/**
 * This class is responsible for holding image objects.
 * They have a position, a name and an image. They may be printed to the screen
 * and user input may be applied.
 *
 * @author sam
 */
public class ImageObject implements IGameObject {

    /** the name of this object.*/
    protected String name;
    /** the image this object holds. */
    protected Image image;
    /** the position of this object. */
    protected Vector2f position;

    /**
     * constructs a new object.
     *
     * @param name name
     * @param image image
     * @param position position at which image should appear
     */
    public ImageObject(String name, Image image, Vector2f position) {
        this.name = name;
        this.image = image;
        this.position = position;
    }

    /**
     * returns the name of this object.
     *
     * @return name
     */
    @Override
    public String getName() {
        return name;
    }

    /**
     *  returns the position of this object.
     *
     * @return position
     */
    @Override
    public Vector2f getPosition() {
        return position;
    }

    /**
     * sets the position of this object.
     *
     * @param position new position
     */
    @Override
    public void setPosition(Vector2f position) {
        this.position = position;
    }

    /**
     * draws the <code>image</code> on the screen with its top left corner
     * at the given x/y coordinates.
     *
     * @param graphics graphics context
     */
    @Override
    public void render(Graphics graphics) {
        image.draw(position.x, position.y);
    }

    /**
     * does nothing at the moment.
     *
     * @param gc GameContainer
     * @param sbg StateBasedGame
     * @param delta milliseconds since last call
     */
    @Override
    public void update(GameContainer gc, StateBasedGame sbg, int delta) {
    }
}