/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sidescroller;

import org.newdawn.slick.Color;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.geom.Shape;
import org.newdawn.slick.geom.Vector2f;

/**
 * @author sam
 */
public class Player extends CollidableImageObject
        implements IMovableObject, IJumpable {

    private int live;
    private int money;
    private boolean isInWater;
    /** counts, how often the player changed state. */
    private int stateChanges;

    /**
     * states, the player can be in.
     */
    public enum STATES {

        JUMPY,
        STICKY,
        STONEY
        // FIXME when in STICKY mode, sometimes
        //  - when first sticking left of block,
        // and then trying to stick under different block,
        //  movement to left or right is not possible.
        // (generaly, sticky mode movement could be nicer...)
    }
    private STATES currentState = STATES.JUMPY;
    /** amount of jumping to do. */
    private float currentJumping;
    public boolean jumping = false;
    /** the last known position of the player. */
    private Vector2f oldPossition;
    private BlockMap map;

    /**
     * constructs new player.
     *
     * @param name name of player
     * @param image image
     * @param position position
     * @param collisionShape collisionShape
     * @param map map
     */
    public Player(String name, Image image, Vector2f position,
            Shape collisionShape, BlockMap map) {
        super(name, image, position, collisionShape);
        this.map = map;
        this.live = GeneralNumbersAndFunctions.STARTLIVE;
        stateChanges = 0;
    }

    /**
     * set whether or not player is in water.
     *
     * @param inWater
     */
    public void setIsInWater(boolean inWater) {
        if (inWater && !isInWater) { // just entered water
            currentJumping = 1;
        }
        isInWater = inWater;
    }

    public STATES getCurrentState() {
        return currentState;
    }

    public void reduceLive(int amount) {
        this.live -= amount;
    }

    public void kill() {
        this.live = -1;
    }

    public boolean isAlive() {
        return this.live >= 0;
    }

    public void changeState(STATES state) {
        currentState = state;
        stateChanges++;
    }

    public int getStateChanges() {
        return stateChanges;
    }

    public void collected(Collectable c) {
        money = money + c.getValue();
    }

    public int getMoney() {
        return money;
    }

    @Override
    public void initiateJump() {
        // only start a jump if player is not jumping already
        // (or if he is in the water)
        if (!jumping || isInWater) {
            // in sticky state do not allow jump from wall
            if (currentState == STATES.STICKY
                    || !map.isRightAboveCollisionBlock(this)) {
                // if in sticky mode and not on ground, jump sideways
                if (map.isRightLeftOfCollisionBlock(this)) {
                    position.x = position.x - 1;
                } else if (map.isRightRightOfCollisionBlock(this)) {
                    position.x = position.x + 1;
                } else if (map.isRightBelowCollisionBlock(this)) {
                    return;
                }
                jumping = true;
                currentJumping = GeneralNumbersAndFunctions.JUMPINONETURN
                        * GeneralNumbersAndFunctions.STICKYSTARTJUMPPERCENT;
                if (isInWater) {
                    currentJumping = currentJumping
                            * GeneralNumbersAndFunctions.WATERSTARTJUMPPERCENT;
                }

            } else if (currentState == STATES.JUMPY) {
                // start normal jump
                jumping = true;
                currentJumping = GeneralNumbersAndFunctions.JUMPINONETURN
                        * GeneralNumbersAndFunctions.JUMPYSTARTJUMPPERCENT;
                if (isInWater) {
                    currentJumping = currentJumping
                            * GeneralNumbersAndFunctions.WATERSTARTJUMPPERCENT;
                }
            } else if (currentState == STATES.STONEY) {
                // start normal jump
                // atm, that means no jump, as JUMPINONETURN is multiplied
                // by 0.
                jumping = true;
                currentJumping = GeneralNumbersAndFunctions.JUMPINONETURN
                        * GeneralNumbersAndFunctions.STONEYSTARTJUMPPERCENT;
                if (isInWater) {
                    currentJumping = currentJumping
                            * GeneralNumbersAndFunctions.WATERSTARTJUMPPERCENT;
                }
            }
        }
    }

    @Override
    public void checkJump() {
        if (jumping) {
            saveOldPosition();
            // continue jump movement
            this.position.set(position.x,
                    position.y + currentJumping);
            // change currentjumping according to gravity
            float gravity = GeneralNumbersAndFunctions.GRAVITY;
            if (isInWater) {
                gravity = gravity / 4f;
            }
            currentJumping += gravity;

            if (map.isRightBelowCollisionBlock(this)
                    && currentJumping <= 0) { // jumped against a block above
                // stop jumping
                currentJumping = 0;
                if (currentState == STATES.JUMPY) {
                    this.resetOnY();
                }
            } else if (map.isRightAboveCollisionBlock(this)
                    && currentJumping >= 0) { // landed
                jumping = false;
                resetOnY();
            }
        }
    }

    public void drop() {
        if (currentState == STATES.STICKY) {
            if (map.isRightBelowCollisionBlock(this)) {
                position.y = position.y + 1;
            }
        }
    }

    public void applyStatebasedMovement() {
        if (map.isOnKillBlock(this)) {
            this.kill();
        }

        switch (currentState) {
            case JUMPY:
                playerWentOfCliff();
                if (!jumping) { // always jump
                    initiateJump();
                }
                break;
            case STICKY:
                if (nextToCollisionBlock()) {
                    currentJumping = 0;
                    jumping = false;
                } else {
                    playerWentOfCliff();
                }
                break;
            case STONEY:
                playerWentOfCliff();
                 break;
            default:
                break;
        }
    }

    private boolean nextToCollisionBlock() {
        return map.isRightBelowCollisionBlock(this)
                // || map.isRightAboveCollisionBlock(this)
                || map.isRightLeftOfCollisionBlock(this)
                || map.isRightRightOfCollisionBlock(this);
    }

    private void playerWentOfCliff() {
        if (!jumping) {
            // apply gravity (in case player went of a cliff, etc)
            if (!map.isRightAboveCollisionBlock(this)) {
                jumping = true; // start jump
                currentJumping = 0; // do not jump upward
                this.position.set(position.x, position.y + 1);
            }
        }
    }

    @Override
    public void moveLeft() {
        moveHorizontally(-GeneralNumbersAndFunctions.SPEED);
    }

    @Override
    public void moveRight() {
        moveHorizontally(GeneralNumbersAndFunctions.SPEED);
    }

    private void moveHorizontally(float distance) {
        // adjust speed acording to state
        float adjustedDistance = 0.0f;
        switch (currentState) {
            case JUMPY:
                adjustedDistance = distance
                        * GeneralNumbersAndFunctions.JUMPYPEEDPERCENT;
                break;
            case STICKY:
                adjustedDistance = distance
                        * GeneralNumbersAndFunctions.STICKYSPEEDPERCENT;
                break;
            case STONEY:
                adjustedDistance = distance
                        * GeneralNumbersAndFunctions.STONEYSPEEDPERCENT;
                break;
            default:
                break;
        }

        if (isInWater) {
            adjustedDistance = distance
                    * GeneralNumbersAndFunctions.WATERSPEEDPERCENT;
        }

        saveOldPosition();
        // make moveHorizontally
        position.set(position.x + adjustedDistance, position.y);
        // check moveHorizontally
        if (map.isColliding(this)) {
            reset();
        }
    }

    public void saveOldPosition() {
        this.oldPossition = this.getPosition().copy();
    }

    @Override
    public void reset() {
        this.position = this.oldPossition;
    }

    public void resetOnX() {
        this.position.x = this.oldPossition.x;
    }

    public void resetOnY() {
        this.position.y = this.oldPossition.y;
    }

    @Override
    public void render(Graphics graphics) {
        super.render(graphics);

        // draw bubble
        Shape collisionShape = this.getCollisionShape();
        Color oldColor = graphics.getColor();
        switch (currentState) {
            case JUMPY:
                graphics.setColor(Color.white);
                break;
            case STICKY:
                graphics.setColor(Color.green);
                break;
            default:
                graphics.setColor(Color.darkGray);
                break;
        }
        graphics.draw(collisionShape);
        graphics.setColor(oldColor);
    }
}